@@ -57,13 +57,14 @@ def encode_integration_key(key):
 
 
 class MammothConnector(object):
-    def __init__(self, email=None, password=None, account_id=None, token=None, api_url=None, api_key=None, api_secret=None):
-
+    def __init__(self, account_id=None, token=None, api_url=None, api_key=None, api_secret=None):
         """
         The main class for handling Mammoth Analytics API
-        :param email: The email of the user
-        :param password: The password of the user
-        :param account_id: Account ID. If not given now, it can be given after `select accounts`
+        Args:
+            :account_id: Account ID.
+            :api_url: API server url
+            :api_key: API key for an account user
+            :api_secret: API secret for an account user
         """
         if not api_url:
             api_url = const.API
@@ -72,18 +73,6 @@ class MammothConnector(object):
 
         if token:
             self.__token = token
-        elif password:
-            response = requests.post(get_url('/login', self.api_url),
-                                     json=dict(email=email, password=password, is_long_term=True))
-            if response.status_code != 200:
-                raise AuthError(response.json()["ERROR_CODE"],
-                                '{0}'.format(response.status_code, response.json()['ERROR_MESSAGE']))
-            self.__token = response.json()['token']
-            response = requests.post(get_url('/login', self.api_url),
-                                     json=dict(email=email, password=password, is_long_term=True))
-            if response.status_code != 200:
-                raise AuthError("username or password is not correct")
-            self.__token = response.json()['token']
         elif api_key and api_secret:
             response = requests.post(get_url('/login', self.api_url),
                                      json=dict(api_key=api_key, api_secret=api_secret, is_long_term=True))
@@ -100,7 +89,7 @@ class MammothConnector(object):
             self.select_account(account_id)
         else:
             raise AuthError("Account ID needed!")
-        log.info("Logged in as {0} into account {1}".format(email, self.__account_id))
+        log.info("Logged in to account {0}".format(self.__account_id))
 
     def __del__(self):
         try:
@@ -242,9 +231,12 @@ class MammothConnector(object):
 
     @handleError
     def delete_dataset(self, ds_id):
+        log.debug(f"Deleting dataset {ds_id}")
         response = self._make_signed_request('delete', '/datasets/{0}'.format(ds_id))
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        log.debug(f"Dataset {ds_id} deleted successfully")
         return future_response
 
     @handleError
@@ -346,9 +338,12 @@ class MammothConnector(object):
 
     @handleError
     def list_batches(self, ds_id):
+        log.debug(f"Fetching batches for ds {ds_id}")
         response = self._make_signed_request('get', '/datasets/{0}/batches'.format(ds_id))
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        log.debug(f"Batches for ds {ds_id} fetched successfully")
         return future_response
 
     @handleError
@@ -364,9 +359,12 @@ class MammothConnector(object):
         :param ds_id: The datasource id
         :return: dictionary containing information on the dataset
         """
+        log.debug("Get dataset for id: {0}".format(ds_id))
         response = self._make_signed_request('get', '/datasets/{0}?INCLUDE_ROW_COUNT={1}'.format(ds_id, include_row_count))
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
+        future_tracking_response = self.track_future_id(future_id)
+        future_response= future_tracking_response['response']
+        log.debug(f"Dataset fetched successfully: {future_response}")
         return future_response
 
     @handleError
@@ -374,6 +372,7 @@ class MammothConnector(object):
         """
         Method to get paginated data for a particular batch of a dataset
         """
+        log.debug(f"Get batch info for dataset {ds_id} and batch {batch_id}")
         paging_params = {
             'columns': columns,
             'condition': condition,
@@ -386,39 +385,53 @@ class MammothConnector(object):
                                              json=paging_params)
 
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        return future_response
+        future_tracking_response = self.track_future_id(future_id)
+        get_batch_response = future_tracking_response['response']
+        log.debug(f"successfully fetched batch {batch_id} for ds {ds_id} :- {get_batch_response}")
+        return get_batch_response
 
     @handleError
     def get_task(self, view_id):
+        log.debug(f"Fetch tasks for view {view_id}")
         response = self._make_signed_request('get', '/workspaces/{0}/tasks'.format(view_id))
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        return future_response.get('tasks')
+        future_req_tracking_response = self.track_future_id(future_id)
+        future_response = future_req_tracking_response['response']
+        task_response = future_response.get('tasks')
+        log.debug(f"Successfully fetched tasks for view {view_id}")
+        return task_response
 
     @handleError
     def add_task(self, view_id, params, wait_till_ready=True):
+        log.debug("Adding task for view_id: {0}".format(view_id))
         r = self._make_signed_request('post', '/workspaces/{0}/tasks'.format(view_id), json={'param': params})
         response = r.json()
-        if wait_till_ready:
-            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        log.debug("Response of adding task to view_id {} is {}".format(view_id, response))
+        if wait_till_ready and response['information']['status'] == 'processing':
+            response = self.track_future_id(response['information']['future_id'])
+        log.debug(f"Task added to view {view_id} successfully with response: {response}")
         return response
 
     @handleError
     def update_task(self, view_id, task_id, params, wait_till_ready=True):
+        log.debug("Updating task for view_id: {0}".format(view_id))
         r = self._make_signed_request('patch', '/workspaces/{0}/tasks/{1}'.format(view_id, task_id),
                                       json={"patch": [{"op": "replace", "path": "params", "value": params}]})
         response = r.json()
-        if wait_till_ready:
-            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        if wait_till_ready and response['information']['status'] == 'processing':
+            response = self.track_future_id(response['information']['future_id'])
+        log.debug(f"Successfully updated task {task_id} in view {view_id} with response: {response}")
         return response
 
     @handleError
-    def delete_task(self, view_id, task_id, wait_till_ready=True):
-        r = self._make_signed_request('delete', '/workspaces/{0}/tasks/{1}'.format(view_id, task_id))
+    def delete_task(self, view_id, task_id, wait_till_ready=True, skip_validation=False):
+        log.debug("Deleting task for view_id: {0}".format(view_id))
+        skip_validation_arg = 'true' if skip_validation else 'false'
+        r = self._make_signed_request('delete', '/workspaces/{0}/tasks/{1}?skip_validation={2}'.format(view_id, task_id, skip_validation_arg))
         response = r.json()
-        if wait_till_ready:
-            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        if wait_till_ready and response['information']['status'] == 'processing':
+            response = self.track_future_id(response['information']['future_id'])
+        log.debug(f"Successfully deleted task {task_id} in view {view_id} with response: {response}")
         return response
 
     @handleError
@@ -480,6 +493,7 @@ class MammothConnector(object):
         Return:
             ds - A ds object.
         """
+        log.debug("Uploading csv file: {0}".format(file_path))
         files = {'file': open(file_path, 'rb')}
         post_data = {}
         if target_datasource_id:
@@ -487,7 +501,7 @@ class MammothConnector(object):
             if replace:
                 post_data['replace'] = "true"
         response = self._make_signed_request('post', '/files', files=files, data=post_data)
-        log.info(response.json())
+        log.debug(f"Response of files: {response.json()}")
         file_id = response.json()['id']
         ds = self.wait_till_file_processing_get_ds(file_id, file_name, ds_with_no_header)
         ds_id = ds['id']
@@ -514,7 +528,7 @@ class MammothConnector(object):
 
     @handleError
     def append_datasets(self, source_dataset_id, target_dataset_id, column_mapping, change_map=None,
-                        new_ds_params=None, replace=False):
+                        new_ds_params=None, replace=False, wait_for_views_to_update=True):
         """
         Method to combine two datasets While appending datasets, target ds gets combined and new rows gets added.At
         this point, ds updated_at will change. Thereafter, the pipeline re-run happens in all workspaces and
@@ -524,6 +538,7 @@ class MammothConnector(object):
         last_updated_at break the loop and return the response
         2. Try until retry_count is less than 50, to avoid stuck issue in worse case
         """
+        log.debug(f"Appending datasets -  source: {source_dataset_id}, target - {target_dataset_id}")
         wksp_data = []
         date_format = '%Y-%m-%dT%H:%M:%SZ'
         params = {
@@ -545,17 +560,21 @@ class MammothConnector(object):
             last_updated_at = res['updated_at']
             # store all workspace id and last_data_updated_at time into a dictionary
             wksp_data.append({'view_id': view_id, 'last_updated_at': last_updated_at})
-
+        log.debug(f"wksp_data before: {wksp_data}")
         # append dataset
         response = self._make_signed_request('post', '/datasets/{0}/batches'.format(target_dataset_id), json=params)
-        if response.json()['STATUS'] == 'SUCCESS':
+        if response.json()['STATUS'] == 'SUCCESS' and wait_for_views_to_update:
+            time.sleep(35)
             self.sync_now(target_dataset_id)
-            time.sleep(5)
+            log.debug(f"Wait for 5 sec to sync ds {target_dataset_id} 's data to views")
+            time.sleep(35)
             for view_details in wksp_data:
                 view_id = view_details['view_id']
                 last_updated_at = view_details['last_updated_at']
+                log.debug(f"Iterating over view {view_details['view_id']} whose last_updated_at stored is {last_updated_at}")
                 res = self.get_view_details(view_id)
                 updated_at = res['updated_at']
+                log.debug(f"Fetched updated at for view {view_details['view_id']} with current value: {updated_at}")
 
                 # convert string to datetime
                 last_updated_at = datetime.strptime(last_updated_at, date_format)
@@ -563,21 +582,26 @@ class MammothConnector(object):
                 retry_count = 0
 
                 while not (updated_at > last_updated_at) and retry_count < MAX_RETRIES:
+
                     # iterate until data_updated_at is not greater than last_data_updated_at
                     retry_count += 1
+                    log.debug(f"Wait for 5 more secs for view to be updated after appending data to dataset")
                     time.sleep(5)
                     res = self.get_view_details(view_id)
                     updated_at = res['updated_at']
+                    log.debug(f"Fetched updated at for view {view_id} in while loop with value: {updated_at}")
                     updated_at = datetime.strptime(updated_at, date_format)
 
                 if retry_count == MAX_RETRIES:
                     raise RuntimeError(f"Max retry limit reached to combine datasets.")
 
             self.sync_now(target_dataset_id)
+            log.debug(f"Another Wait for 15 secs to sync ds {target_dataset_id} 's data to views")
             time.sleep(15)
             r = self._make_signed_request('get', '/datasets/{0}/batches'.format(target_dataset_id))
             future_id = r.json().get('future_id')
-            future_response = self.track_future_id(future_id)
+            future_tracking_response = self.track_future_id(future_id)
+            future_response = future_tracking_response['response']
             self.wait_till_ds_ready(target_dataset_id)
             return future_response
 
@@ -590,13 +614,14 @@ class MammothConnector(object):
         :param file_id:
         :return: A dataset dictionary
         """
+        log.debug(f"Waiting till file: {file_id} is processing")
         retry_count = 0
         while retry_count < MAX_RETRIES * 4:
             response = self._make_signed_request('get', '/files/{0}'.format(file_id))
             file_info = response.json()['file']
             status = file_info['status']
             if status != 'processing':
-                log.info(file_info)
+                log.debug(f"Current info of file - {file_id} is {file_info}")
                 break
             else:
                 time.sleep(RETRY_DELAY_IN_SEC)
@@ -605,11 +630,12 @@ class MammothConnector(object):
             raise RuntimeError(f"Max time limit reached to get processed file .")
         response = self._make_signed_request('get', '/files/{0}'.format(file_id))
         file_info = response.json()['file']
+        log.debug(f"File info finally: {file_info}")
         final_ds_id = file_info["additional_data"].get("final_ds_id")
         append_to_ds_id = file_info["additional_data"].get("append_to_ds_id")
         if final_ds_id is None and file_name is None:
             raise UnknownError(f"File name required for .xls, .xlsx, .zip file extensions.")
-        # wait till the file reflects in resources to get dataset id by filtering resources
+        log.debug(f"Wait 5 seconds till the file reflects in resources to get dataset id by filtering resources")
         time.sleep(5)
         if final_ds_id is None and file_name is not None:
             resources = self.list_resources()
@@ -633,6 +659,8 @@ class MammothConnector(object):
 
     @handleError
     def _wait_for_ds_status(self, ds_id, status, check_for_status_equality=True):
+        intention = 'equal' if check_for_status_equality else 'not_equal'
+        log.debug(f"Waiting for dataset {ds_id} status to {intention} {status}")
         retry_count = 0
         while ds_id and retry_count < MAX_RETRIES * 4:
             ds = self.get_dataset(ds_id)
@@ -745,11 +773,13 @@ class MammothConnector(object):
         return response.json()
 
     def reorder_tasks(self, view_id, param, wait_till_ready=True):
+        log.debug("Reordering tasks for view_id: {0}".format(view_id))
         r = self._make_signed_request('patch', '/workspaces/{0}/tasks'.format(view_id),
                                       json={"patch": [{"op": "replace", "path": "tasks", "value": param}]})
         response = r.json()
-        if wait_till_ready:
-            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
+        if wait_till_ready and response['information']['status'] == 'processing':
+            response = self.track_future_id(response['information']['future_id'])
+        log.debug(f"Successfully reordered tasks in view {view_id} with response: {response}")
         return response
 
     @handleError
@@ -759,6 +789,7 @@ class MammothConnector(object):
         Then calls future request tracker method to get future requets's response
         Finally returns the data of the wksp as per params passed
         """
+        log.debug(f"Get data for view {view_id}")
         paging_params = {
             'columns': columns,
             'condition': condition,
@@ -769,9 +800,11 @@ class MammothConnector(object):
         response = self._make_signed_request('post', 'workspaces/{workspace_id}/data'.format(workspace_id=view_id),
                                              json=paging_params)
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        log.debug("The future response is {}".format(future_response))
-        return future_response['data']
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        view_data = future_response['data']
+        log.debug(f"Successfully fetched data for view  {view_id} : {future_response}")
+        return view_data
 
     @handleError
     def run_transform(self, view_id, param):
@@ -780,17 +813,21 @@ class MammothConnector(object):
 
     @handleError
     def get_view_details(self, view_id):
+        log.debug(f"Get details for view {view_id}")
         r = self._make_signed_request('get', '/workspaces/{0}'.format(view_id))
         future_id = r.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        return future_response
+        future_tracking_response = self.track_future_id(future_id)
+        log.debug(f"Successfully fetched details for view {view_id}")
+        return future_tracking_response['response']
 
     @handleError
     def set_view_properties(self, view_id, properties):
+        log.debug(f"Set properties for view {view_id}")
         r = self._make_signed_request('post', '/workspaces/{0}'.format(view_id), json=properties)
         future_id = r.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        return future_response
+        future_tracking_response = self.track_future_id(future_id)
+        log.debug(f"Successfully updated properties for view {view_id}")
+        return future_tracking_response['response']
 
     @handleError
     def wait_for_view_to_finish_processing(self, view_id):
@@ -895,11 +932,14 @@ class MammothConnector(object):
 
     @handleError
     def list_actions(self, view_id):
+        log.debug(f"Fetching actions for view {view_id}")
         response = self._make_signed_request('get', '/workspaces/{0}/actions'.format(view_id))
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
-        log.debug("List actions : {0}".format(response))
-        return future_response['triggers']
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        actions = future_response['triggers']
+        log.debug(f"Actions fetched for view {view_id} are: {actions}")
+        return actions
 
     @handleError
     def delete_action(self, view_id, action_id):
@@ -1074,6 +1114,7 @@ class MammothConnector(object):
         and returns response as JSON object when status is not in
         processing (i.e. SUCCESS/FAILED)
         """
+        log.debug("Tracking fututre request with id: {}".format(future_id))
         if not isinstance(future_id, int):
             raise RuntimeError(f"Future id should be a numeric value")
 
@@ -1085,13 +1126,14 @@ class MammothConnector(object):
             resource_resp = self._make_signed_request('get', f'/future/{future_id}')
             resource_resp_json = resource_resp.json()
             future_object = resource_resp_json['future']
+            response = future_object
             if future_object['status'] != 'processing':
-                response = future_object['response']
                 break
             retry_count += 1
 
         if retry_count == MAX_RETRIES:
             raise RuntimeError(f"Max limit reached to future request response.")
+        log.debug("Response from future request: {}".format(response))
         return response
 
     @handleError
@@ -1122,7 +1164,7 @@ class MammothConnector(object):
         :return - List of batch column/s keys
         """
         batch_column_keys = []
-        metadata = self.get_dataset(ds_id)['meta_data']
+        metadata = self.get_dataset(ds_id)['metadata']
         for value in metadata:
             if value['internal_name'] in RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS.keys():
                 batch_col_key = RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS[value['internal_name']]
@@ -1166,14 +1208,18 @@ class MammothConnector(object):
 
     @handleError
     def delete_publish(self, view_id, publish_trigger_id):
+        log.debug(f"Deleting publish trigger with id: {publish_trigger_id} from view {view_id}")
         response = self._make_signed_request('delete', f'/workspaces/{view_id}/publish/{publish_trigger_id}')
 
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        log.debug(f"Deleted publish from view {view_id} successfully")
         return future_response
 
     @handleError
     def regenerate_publish_password(self, view_id, publish_odbc_type):
+        log.debug(f"Regenerating password for publish in view {view_id}")
         param = {
             "patch": [{
                 "op": "replace", "path": "reset_password",
@@ -1187,7 +1233,9 @@ class MammothConnector(object):
                                              json=param)
 
         future_id = response.json()['future_id']
-        future_response = self.track_future_id(future_id)
+        future_tracking_response = self.track_future_id(future_id)
+        future_response = future_tracking_response['response']
+        log.debug(f"Successfully regenerated password for publish in view {view_id}")
         return future_response['password']
 
     @handleError
@@ -1212,10 +1260,13 @@ class MammothConnector(object):
         :param destination_folder_id - id of the folder to which files are moving
         :param params - params which contains files to be moved
         """
+        log.debug(f"Moving file to folder: {destination_folder_id}")
         response = self._make_signed_request('post', '/labels/{0}/resources'.format(destination_folder_id), json=params)
         future_id = response.json()['future_id']
         if wait_till_ready:
-            future_response = self.track_future_id(future_id)
+            future_tracking_response = self.track_future_id(future_id)
+            future_response = future_tracking_response['response']
+        log.debug(f"File  moved successfully")
         return future_response
 
     @handleError
@@ -1323,7 +1374,8 @@ class MammothConnector(object):
                                              json={"limit": 10000})
         future_id = response.json()['future_id']
         if wait_till_ready:
-            future_response = self.track_future_id(future_id)
+            future_tracking_response = self.track_future_id(future_id)
+            future_response = future_tracking_response['response']
         return future_response
 
     @handleError
@@ -1336,7 +1388,8 @@ class MammothConnector(object):
         response = self._make_signed_request('delete', '/workspaces/{0}/preview-mode'.format(view_id))
         future_id = response.json()['future_id']
         if wait_till_ready:
-            future_response = self.track_future_id(future_id)
+            future_tracking_response = self.track_future_id(future_id)
+            future_response = future_tracking_response['response']
         return future_response
 
     @handleError
@@ -1528,7 +1581,8 @@ class MammothConnector(object):
         r = self._make_signed_request('post', '/workspaces/{0}/data/query'.format(view_id), json=params)
         response = r.json()
         if wait_till_ready:
-            response = self.track_future_id(response['future_id'])
+            future_tracking_response = self.track_future_id(response['future_id'])
+            response = future_tracking_response['response']
         return response
 
     @handleError
@@ -1595,7 +1649,8 @@ class MammothConnector(object):
         response = response.json()
         if wait_till_ready:
             future_id = response['future_id']
-            response = self.track_future_id(future_id)
+            future_tracking_response = self.track_future_id(future_id)
+            response = future_tracking_response['response']
         return response
 
     @handleError
@@ -1667,6 +1722,13 @@ class MammothConnector(object):
         return response.json()
 
     @handleError
+    def update_pipeline_inbound_updates(self, view_id, disable=False):
+        """
+        Method to snooze/unsnooze pipeline updates
+        """
+        response = self._make_signed_request('post', 'workspaces/{0}/inbound-updates'.format(view_id), json={'disable': disable})
+        return response.json()
+        
     def add_metric(self, view_id, params, condition=None, limit=None, wait_till_ready=True):
         """
         Method to add a metric
@@ -1730,3 +1792,58 @@ class MammothConnector(object):
         if retry_count == MAX_RETRIES:
             raise RuntimeError(f"Max limit reached to add a metric")
         return response
+
+    @handleError
+    def toggle_draft_mode(self, view_id, mode):
+        """
+        Method to enter or exit draft mode for given view
+        Returns:
+            response: return response json of API request contains account details
+        """
+        res = self._make_signed_request('post', '/workspaces/{0}/draft-mode'.format(view_id),
+                                        json={'draft_operation': mode})
+        response = res.json()
+        return response
+    
+    @handleError
+    def modify_password(self, params):
+        """
+        Method to modify user password
+        Args:
+            params: modify password post request params
+        """
+        response = self._make_signed_request('post', '/modify-password', json=params)
+        return response.json()
+
+    @handleError
+    def suspend_task(self, view_id, task_id, skip_validation=False, wait_till_ready=True):
+        """
+        method to suspend action
+        :param
+            view_id: workspace id
+            task_id: task id
+        """
+        r = self._make_signed_request('patch', '/workspaces/{0}/tasks/{1}'.format(view_id, task_id),
+                                      json={"patch": [{"op": "suspend", "path": "status",
+                                                       "value": {"skip_validation": skip_validation}}]})
+        response = r.json()
+        if wait_till_ready:
+            future_id = response.get('information').get('future_id')
+            response = self.track_future_id(future_id)
+        return response
+
+    @handleError
+    def restore_task(self, view_id, task_id, skip_validation=False):
+        """
+        method to restore action
+        :param
+            view_id: workspace id
+            task_id: task id
+        """
+        r = self._make_signed_request('patch', '/workspaces/{0}/tasks/{1}'.format(view_id, task_id),
+                                      json={"patch": [
+                                          {"op": "restore", "path": "status",
+                                           "value": {"skip_validation": skip_validation}}]})
+        response = r.json()
+        return response
+    
\ No newline at end of file
